const test = require('tape')
const helpers = require('./helpers')

test('compiles numbers', t => {
  t.plan(1)

  const result = helpers.compile('5')
  t.equal(result, '5')
})

test('compiles strings', t => {
  t.plan(1)

  const result = helpers.compile('"hello world"')
  t.equal(result, 'hello world')
})

test('compiles lambdas', t => {
  t.plan(1)

  const result = helpers.compile('(lambda (x) (* x x))')
  t.equal(result, '<lambda>')
})

test('compiles applications that evaluate to numbers', t => {
  t.plan(1)

  const result = helpers.compile('((lambda (x) (+ x 1)) 5)')
  t.equal(result, '6')
})

test('simple element', t => {
  t.plan(1)
  const result = helpers.compile('(div :class "container" "Lorem ipsum dolor sit amet")')
  t.equal(result, '<div class="container">Lorem ipsum dolor sit amet</div>')
})

test('nested element', t => {
  t.plan(1)
  const result = helpers.compile('(div (span "Lorem ipsum dolor sit amet"))')
  t.equal(result, '<div><span>Lorem ipsum dolor sit amet</span></div>')
})

test('self closing elements', t => {
  t.plan(1)
  const result = helpers.compile('(img :src "logo.png")')
  t.equal(result, '<img src="logo.png">')
})
